/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.agents;

import java.io.*;
import java.net.*;
import java.util.*;

import osmq.util.*;
import osmq.mque.*;
import osmq.logs.Logs;
import osmq.net.SocketStreamFilter;

final class ReaderConnectionHandler extends Threaded
   {
   // Thread priority
   final public static int SUBSCRIBER_PRIORITY     = Thread.NORM_PRIORITY - 1;
   final public static int SUBSCRIBER_YIELD_COUNT  = 2;
   final public static int PIPELINE_THRESHOLD     = 1000;

   private SocketStreamFilter      _connection = null;
   String                          _ipstring;

   private ReaderConnectionManager _cm;
   private Accumulator             _localcount = new Accumulator();

   // all inbound messages are placed on this sorting queue
   MQue _pipeline;

   private int messages = 1;
   private byte[] inarray;

   private char firstchar;

   public ReaderConnectionHandler(MQue pipeline, ReaderConnectionManager cm)
      {
      _cm = cm;
      cm.getHandlers().addElement(this);
      _pipeline = pipeline;

      }

   public void setStreamFilter(SocketStreamFilter sw)
      {
      _connection = sw;
      _ipstring = sw.getInetAddress().getHostAddress();
      }

   public SocketStreamFilter getStreamFilter()
      {
      return _connection;
      }

   public String getRemoteIPString()
      {
      return _ipstring;
      }

   public String readln() throws IOException
      {
      return _connection.readln();
      }

   public byte[] readBytes() throws IOException
      {
      return _connection.readBytes();
      }

   public void writeBytes(byte[] b) throws IOException
      {
      _connection.writeBytes(b);
      }

   /**
     * Writes a String value to the socket stream.
     */
   public void println(String s) throws IOException
      {
      _connection.println(s);
      }

    protected static void killGroup(Vector v)
      {
      try
         {
         Iterator i = v.iterator();
         if(i != null)
           while(i.hasNext())
             {
             ReaderConnectionHandler sh = (ReaderConnectionHandler) i.next();
             if(sh != null)
               {
               try{sh.stop();}
               catch(Exception e){}
               }
            }
         i = null;
         }
      catch(Exception e)
         {
         Logs.err.println("Error in killGroup " + e.getMessage());
         }
      }

   final protected void initializer() throws Exception
      {
      _initializer();
      }

   final private void _initializer() throws Exception
      {
      this.setPriority(SUBSCRIBER_PRIORITY);
      this.setYieldCount(SUBSCRIBER_YIELD_COUNT);
      }

   final protected void iterativeProcess() throws Exception
      {
      _iterative();
      }


   final private void _iterative() throws Exception
      {
      try{
         if(_cm.isShutdownAll())
            return;
         // Pause reads if connection manager is in a paused state.
         // Similarly, pause if pipeline is beyond capacity.
         if(_cm.isPauseHandlers() ||
            _pipeline.size() >= (2 * PIPELINE_THRESHOLD))
            {
            Threaded.sleepTenths(5);
            return;
            }

         inarray = readBytes();

         if(inarray != null && inarray.length > 1)
            {
            _pipeline.writeBytes(inarray);

            _cm.getAccumulator().increment();
            _localcount.increment();
            messages++;
            }
         else
            Logs.err.println("Input message not > 1 character");
         }

      catch(Exception e)
         {
         Logs.err.println("Exception in PublisherHandler " + e.getMessage());
         throw e;
         }
      }

   public long getLocalCount()
       {return _localcount.getCount();}


   protected void cleanup()
      {
      _cm.getHandlers().removeElement(this);
      if(_connection != null)
        {
        try {_connection.close();}
        catch(Exception e) {System.out.println(e);}
        finally{_connection = null;}
        }
      }
  }

