/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */



package osmq.io;

import java.net.*;
import java.io.*;
import java.util.*;
import java.math.*;
import osmq.util.SpecialChars;


/*
 * An OutputStream filter that supports writing Strings and ASCII byte arrays
 * to an output stream.  To write non-ASCII bytes to a stream, use the
 * PacketOutputStream class. 
 * <p>The corollary StringInputStream is able to read logical Strings and
 * arrays of bytes as they were written.
 * <p> This class is thread safe.
 */
public class StringOutputStream extends FilterOutputStream implements WriterStream
   {
   public StringOutputStream(OutputStream out)
      {
      super(out);
      }

   public void writeNull() throws IOException
      {
      synchronized(this)
         {
         out.write(SpecialChars.STRING_TERMINATOR_BYTE);
         }
      }

   public void writeBytes(byte[] value) throws IOException
      {
      synchronized(this)
         {
         if(value != null && value.length > 0)
             {
             out.write(value, 0, value.length);
             out.write(SpecialChars.STRING_TERMINATOR_BYTE);
             }
          }
      }

   /**
     *
     * Writes an integer (size) to the output stream as 4 bytes.
     *
     * @param      b   the data.
     * @exception  IOException  if an I/O error occurs. In particular, 
     *             an <code>IOException</code> may be thrown if the 
     *             output stream has been closed
     */
   public void println(String s) throws IOException
       {
       synchronized(this)
          {
          if(s.length() > 0)
             {
             out.write(s.getBytes(), 0, s.length());
             out.write(SpecialChars.STRING_TERMINATOR_BYTE);
             }
          }

       }
   }
