/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.logs;

import java.io.*;
import osmq.util.DateStamp;

public class PrintWriterLog extends PrintWriter implements Log
  {
  private Exception ex = null;
  private boolean dated = false;
  private boolean timed = false;
  
  /**
    * Turn a standard OutputStream into a DataOutputStream
    */
  public PrintWriterLog(OutputStreamWriter writer)
     {
     // do not buffer regular streams
     super(new BufferedWriter(writer));
     }

  /**
    * Create an output file as a DataOuputStream, and append if
    * one exists with the filename.
    */
  public PrintWriterLog(String filename) throws IOException
    {
    this(filename, true);
    }
    
  public PrintWriterLog(OutputStream stream)
     {
     super(new OutputStreamWriter(stream));
     }
     
  public PrintWriterLog(String filename, boolean append) throws IOException
    {
    super(new OutputStreamWriter(
            new FileOutputStream(filename, append)));
    }

  /**
    * Returns Exception from last unsuccessful println() call
    */
  public Exception getException()
     {
     return ex;
     }

  /**
    * @returns true if successful, else false
    */
  public synchronized void println(String v)
    {
    if(dated)
          super.println(DateStamp.getFormattedStamp() + " > " + v);
    else
       if(timed)
          super.println(DateStamp.getTimeStamp() + " > " + v);
       else
          super.println(v);
    }

  public void setDated(boolean b)
    {dated = b;}
  public boolean getDated()
    {return dated;}

  public void setTimed(boolean b)
    {timed = b;}
  public boolean getTimed()
    {return timed;}

  protected void finalize() throws Throwable
     {
     close();
     super.finalize();
     }
  }
