/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

  
package osmq.messages;

import osmq.records.DataRecordBean;

/**
  * Each thread should allocate a seperate MessageFactory instance;
  */
class MessageFactoryUtil
   {
   private static MessageFactoryUtil _mb;
   private MessageFactoryUtil(){}

   static MessageFactoryUtil getSingleton()
      {
      if(_mb == null)
         {
         synchronized(MessageFactoryUtil.class)
            {
            if(_mb == null)
               _mb = new MessageFactoryUtil();
            }
         }
      return _mb;
      }


   ERMessage createNotificationMessage(int size, String topic)
      {
      // First crate a message with the correct number of body elements
      ERMessage m = new ERMessageBean(size);

      // Set its topic
      m.setTopic(topic);

      // Identify it as a notification message
      m.setType(MessageType.NOTIFICATION);
      
      return m;
      }
      
   /**
     * Creates an ERMessage or BytePacketMessage, depending on
     * the value of the message format found in the byte array.
     */
   synchronized Message createMessage(byte[] value) throws MessageException
      {
      Message m;
      try
         {
         // Test for a valid byte array
         if(value == null || value.length < MessageAttributes.TOTAL_ATTRIBUTES + 1)
            throw new MessageException("Invalid byte sting in MessageFactoryUtil.createMessage(byte[]))");

         // Find position of the delimiter immediately following last header element.
         int hdrlgh = getDelimiterIndex(value, MessageAttributes.TOTAL_ATTRIBUTES + 1);

         // If the position is not found, getDelimiterIndex returned a value of -1.
         if(hdrlgh == -1)
             throw new MessageException("Invalid message in MessageFactoryUtil.createMessage(byte[])");


         // Create a header instance from the first portion of the byte array, minus the trailing delimiter
         MessageHeader header = new MessageHeader(new String(value, 0, hdrlgh-1));

         // If the message type is DATASET, the delimiter byte immediately after the
         // header portion of the message is part of the body.  Otherwise, it is ignored.

         if(header.getFormat() == MessageFormat.DATASET)
            hdrlgh--;

         // Allocate a byte array to hold the body portion of the message
         byte[] clientbytes = new byte[value.length - hdrlgh];

         // Fill the body array values from the source array
         System.arraycopy(value, hdrlgh, clientbytes, 0, clientbytes.length);

         // Based on the message format attribute in the header,
         // create the appropriate message class from the header object and body value
         if(header.getFormat() == MessageFormat.DATASET)
             m = new ERMessageBean(header, clientbytes);
         else
             m = new BinaryMessageBean(header, clientbytes);

         // Return the message to the caller
         return m;
         }
         
      catch(Exception e){throw new MessageException(e.getMessage());}
      }

   ERMessage createMessage(int sz)
      {
      return new ERMessageBean(sz);
      }

   ERMessage createERMessage(String v)
     {
     return new ERMessageBean(v);
     }

   Message createImmutableMessage(String v)
     {
     ERMessage m = new ERMessageBean(v);
     m.setImmutable();
     return m;
     }

   Message createImmutableMessage(byte[] b) throws MessageException
     {
     try{
        Message m = createMessage(b);
        m.setImmutable();
        return m;
        }
     catch(Exception e)
        {throw new MessageException(e.getMessage());}
     }

   /**
     * Returns the nth occurance of the delimiter in a delimited string.  The
     * first character in the string is considered the delimiter.
     */
   private int getDelimiterIndex(byte[] value, int occurance)
      {
      int maxposition = value.length;  // max is size of the array
      byte delimiter = value[0];       // first char is the delimiter
      int count = 0, pos = 0;          //

      for(pos=0; pos < maxposition && count < occurance; ++pos)
        {
        if(value[pos] == delimiter)
           ++count;
        }
      return(pos < maxposition) ? pos : -1;
      }

   }