/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.net;

import java.net.*;
import java.io.*;
import java.util.*;
import java.text.*;
import osmq.mcns.*;

/**
  * Sends a "where is" request to a multicast group/port, and waits for a service
  * to respond with its host/port.
  * Uses a singleton to access, so there are not multiple multicast instances
  * for a single application.
  */

public class DiscoveryClient extends MCRequestor
  {
  private static DiscoveryClient singleton;

  private DiscoveryClient()
     {
     setExceptionOnNoResponse(false);
     }

  protected int getDefaultChannel()
     {return MCNS.DISCOVERY_CHANNEL;}

  public static DiscoveryClient getSingleton()
     {
     if(singleton == null)
        singleton = new DiscoveryClient();
     return singleton;
     }

  public boolean isActiveService(String servicename)
     {
     try
        {
        return sendRequest(servicename.toUpperCase()) != null;
        }
     catch(IOException e)
        {return false;}
     }

  /**
    * locate service name embedded in es, and set the port and address
    * of es.
    */
  public RemoteService locate(String servicename) throws IOException
     {

     RemoteService _rs = new RemoteService(servicename);
     if(!_rs.portIsSet())
            buildResult(_rs, sendRequest(_rs.getServiceID().toUpperCase()));
     return _rs;
     }

  /**
    * Locate service name embedded in es, and set the port and address
    * of es.
    */
  public RemoteService locate(RemoteService es) throws IOException
     {
     try{
        if(!es.portIsSet())
            buildResult(es, sendRequest(es.getServiceID().toUpperCase()));
        return es;
        }
     catch(IOException e)
        {
        System.err.println("Error locating service in DiscoveryClient : " +
                 e.getMessage());
        throw e;
        }
     }

  private RemoteService buildResult(RemoteService es, String source) throws IOException
     {
     if(source != null)
        {
        // null the address and port number in the result set
        es.clearAddress();

        StringTokenizer st = new StringTokenizer(source);
        if (st.hasMoreTokens())
           {
           // The first token in the string should match the service ID
           // in the RemoteService instance
           if(!es.getServiceID().equals(st.nextToken()))
              return es;
           }
        else
           return es;

        if (st.hasMoreTokens())
           es.setHostName(st.nextToken());
        else
           return es;

        if(st.hasMoreTokens())
           es.setPortID(Integer.parseInt(st.nextToken()));
        else
           return es;

        if(st.hasMoreTokens())
           es.setStreamMode(st.nextToken());

        return es;
        }
     return es;
     }

  }

