/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.net;

import java.io.*;
import java.net.*;
import java.util.*;
import osmq.mcns.*;

class RemoteServiceAttrib
   {
   private String name;
   private int port;

   RemoteServiceAttrib(String name, int port)
         {
         this.name = name;
         this.port = port;
         }
   String getName(){return name;}
   int getPort(){return port;}
   }

/**
  * A DiscoverySocket is a standard socket implementation that
  * can use dynamic discovery to determine the host
  * address and port.
  */
public class DiscoverySocket extends Socket
   {
   private RemoteService id = null;

   /**
     * Attempts to attach to the host identified in the HostIdentifier.
     */
   DiscoverySocket(RemoteService identifier) throws IOException
      {
      this(resolveAddress(identifier));
      id = identifier;
      }

   /**
     * Attempts to attach to the remote service.
     */
   public DiscoverySocket(String servicename) throws IOException
      {
      this(DiscoverySocket.fromServiceName(servicename));
      }
   
   private DiscoverySocket(RemoteServiceAttrib v) throws IOException
      {
      this(v.getName(), v.getPort());
      }
      
   private static RemoteService fromServiceName(String servicename)
      {return new RemoteService(servicename);}

      
   public boolean isBinaryMode()
      {
      return id != null && id.isBinaryMode();
      }

   public DiscoverySocket(String hostname, int portid) throws IOException
      {
      super(hostname, portid);
      }

    private static RemoteServiceAttrib resolveAddress(RemoteService es) throws IOException
      {
      if(!es.portIsSet())
          es = findService(es);
      return new RemoteServiceAttrib(es.getHostName(), es.getPortID());
      }

   private static RemoteService findService(RemoteService es) throws IOException
      {
      RemoteService updated;
      if(es.portIsSet())
         return es;
      if(es.getServiceID() == null)
         throw new NetException ("Service Name not set");
      DiscoveryClient.getSingleton().locate(es);
      if(!es.portIsSet())
      // test and throw exception if could not locate host
         {
         NetException mr = new NetException("Could not locate service " + es.getServiceID());
         throw mr;
         }
      return es;
      }
   }
