/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.net;

import java.io.IOException;

public class RemoteService
  {
  final private static char BINARY_MODE = 'B';

  private String _serviceid = null;
  private String _streamMode = null;
  private int    _port = 0;
  private String _hostname = null;

  public RemoteService(){}

  public RemoteService(String serviceid)
      {
      setServiceID(serviceid);
      }

  public RemoteService(String hostname, int port)
      {
      setHostName(hostname);
      setPortID(port);
      }

   public boolean isNamed()
      {
      return _serviceid != null;
      }

   public String getServiceID()
      {
      return _serviceid;
      }

   public void setServiceID(String serviceid)
      {
      _serviceid = serviceid.toUpperCase();
      }

   public boolean isBinaryMode()
      {
      return _streamMode != null &&
         _streamMode.charAt(0) == BINARY_MODE;
      }

   public String getStreamMode()
      {
      return _streamMode;
      }

   public  void setStreamMode(String streamMode)
      {
      _streamMode = streamMode;
      }

   public void clearAddress()
      {
      setPortID(0);
      setHostName(null);
      }
      
   public void setPortID(int port)
      {
      _port = port;
      }
      
   public int getPortID()
      {
      return _port;
      }
      
   public boolean portIsSet()
      {
      return _port > 0;
      }

   public boolean hostNameIsSet()
      {
      return _hostname != null && _hostname.length() > 0;
      }
      
   public void setHostName(String hostname)
      {
      _hostname = hostname;
      }
      
   public String getHostName()
      {
      return _hostname;
      }

   public String toString()
      {
      if(hostNameIsSet() && portIsSet())
         return new String(getHostName() + ":" + getPortID());
      else
         return "";
      }
  }