/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.net;

import java.io.*;
import java.net.*;
import java.util.*;


import osmq.util.Threaded;
import osmq.util.Accumulator;
import osmq.util.ExceptionEventBean;
import osmq.util.ExceptionListener;
import osmq.logs.Logs;

/**
  * Base class for socket server class that listens for publisher and subscriber
  * connections, passing them to a dedicated thread.
  */
abstract public class ServerSocketProcessor extends Threaded
   {
   private ExceptionEventBean eebean = new ExceptionEventBean();
   private ServerSocket _ss = null;
   private Socket newWorker = null;
   private int portID = 0;
   private boolean binarymode;

   public ServerSocketProcessor(ServerSocket ss) throws IOException
      {_ss = ss;}

   public ServerSocketProcessor()
      {}

   abstract protected void launchNewWorker(Socket workersock) throws IOException;

   /**
     * if <i>binary</i> is <i>true</i>, the socket will
     * be opened in binary content mode.  Otherwise, socket
     * will be opened in text mode.
     */
   public void setBinarySocketMode(boolean binary)
      {binarymode = binary;}
   public boolean isBinarySocketMode()
      {return binarymode;}

   public void setPortID(int v)
     {portID = v;}
   public int getPortID()
     {return portID;}

   public void addExceptionListener(ExceptionListener listener)
      {
      eebean.addExceptionListener(listener);
      }

   public void removeExceptionListener(ExceptionListener listener)
      {
      eebean.removeExceptionListener(listener);
      }

   protected void initializer() throws Exception
      {
      if(getPortID() == 0)
         throw new IOException("Invalid Port ID");
      if(_ss == null)
         _ss = new ServerSocket(getPortID());
      }


   protected void iterativeProcess() throws Exception
     {
     // Accept a new socket connection
     try{newWorker = _ss.accept();}

     // If there was a socket exception, close the
     // newly created socket
     catch(SocketException v)
        {

        try{newWorker.close();}
        catch(Exception e){} // close the worker socket

        Logs.err.println("Error in Server.accept() within ServerSockerProcessor() "
                         + v.getMessage());
        return;
        }

      catch(Exception v)
           {
           Logs.err.println("Error in ServerSocketProcessor: " +
                              v.getMessage());
           return;
           }

     try{
        launchWorker();
        }
     catch(Exception e)
        {
        try{newWorker.close();}
        catch(Exception ignore){}
        Logs.err.println("Error launching worker in ServerSocketProcessor " +
                         e.getMessage());
        eebean.notifyListeners(e);
        // implicit return without throwing exception
        }
     }


   private void launchWorker() throws Exception
      {
      SocketStreamFilter bs = new SocketStreamFilter(newWorker, isBinarySocketMode());
      launchNewWorker(newWorker);
      }

   protected void cleanup()
      {
      try
        {
        // close the listening socket
        if(_ss != null)
           _ss.close();
        }
      catch(Exception e) {}
      finally {_ss = null;}
      }
}


