/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.pulsar;


import java.net.*;
import java.io.*;

import osmq.util.*;
import osmq.util.Threaded;
import osmq.mcns.*;
import osmq.records.DataRecordListener;
import osmq.records.DataRecordBean;

/** Aggregate class used by a service's client to identify if and when a service fails.
  * The service being monitored must employ a MulticastPulsar at the standard "pulsar" frequency.
  * The client class registers a function to be called if / when the service fails.
  */

public class DatasetPulsarListener extends Threaded
  {
  private ExceptionListener exlistener;
  private DataRecordListener dsl;
  private boolean shouldExit = false;
  private boolean timeoutfatal = true;
  private MulticastListener listener;

  private int listenport = MCNS.PULSAR_CHANNEL;
  private boolean servicefailure = false;

  public static int buffer_size = 1024;

  public static byte buffer[] = new byte[buffer_size];

  public DatasetPulsarListener()
    {
    listener = MulticastListener.createListener();
    listener.setTimeoutSecs(Threshold.TIMEOUT_THRESHOLD);
    }

  public void setTimeoutSeconds(int s)
    { listener.setTimeoutSecs(s);}
  public int getTimeoutSeconds()
    {return listener.getTimeoutSecs();}

  public void setPulsarPort(int p)
    {listenport = p;}
  public int getPulsarPort()
    {return listenport;}

  public  void setDataRecordListener(DataRecordListener l)
    {dsl = l;}

  public  DataRecordListener getDataRecordListener()
    {return dsl;}

 /**
   * Causes the ServicePulsarMonitor to begin listening for the pulse of a service.
   * If it detects a pulse lapse greater than a timeout value
   * (timeout value defaults to Multicasters.PulsarAlarmThreshold in seconds), it
   * will notify all classes that have registered as FatalEventListener instances.
   */
  final protected void initializer()  throws Exception
    {
    if(getDataRecordListener() == null)
       throw new Exception("DataRecordListener not set in DatasetPulsarListener.class");
    servicefailure = false;
    listener.setChannel(listenport);
    System.out.println("Starting DatasetPulsarListener for port " + listenport);
    }

  final protected void iterativeProcess()  throws Exception
     {
     String rv;
     listener.getNextPacket();
          
     if(listener.isTimedOut())
        {
        // NO PULSE OF ANY SORT ON THE PORT
        servicefailure = true;
        System.out.println("DatasetPulsar timeout");
        if(getExceptionListener() != null)
           getExceptionListener().onException(new Exception("DatasetPulsar timeout"));
        throw new Exception("DatasetPulsar timeout");
        }
             
     // VERIFY PULSE FROM THE CORRECT SERVICE
     getDataRecordListener().onDataRecord(new DataRecordBean(listener.getRemoteData()));
     }

  
  final protected void cleanup()
     {
     listener.close();
     }

  public void setExceptionListener(ExceptionListener listener)
     {
     exlistener = listener;
     }

  public ExceptionListener getExceptionListener()
     {
     return exlistener;
     }

  }
