/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.sample;

import java.util.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.IOException;
import java.text.DecimalFormat;

import osmq.mcns.*;
import osmq.agents.NetMessageWriter;
import osmq.util.*;
import osmq.records.DataRecord;
import osmq.mque.*;
import osmq.messages.*;


public class TestMWriter implements ExceptionListener
   {
   private int TEN_MILLION = 10000000;

   public TestMWriter(){}
   public void onException(Exception e)
      {
      System.out.println("Exception in TestPub " + e.getMessage());
      }

   public void run()
      {
      NetMessageWriter pb;
      // Create a message with 10-field capacity, and with a topic
      ERMessage message = MessageFactory.createNotificationMessage(10, "MYTOPIC");

      // Set a unique key value (so that is can be written to a database)
      message.setKey("KEY");

      // Define the type of transaction to be used for the message data
      message.setTransactionAsChar(MessageAttributes.TRANS_ADD);

      // Define the various user-defined field values
      message.setString(1,"FIRST");
      message.setString(2, "SECOND");
      message.setOriginator("ORIGINATOR");


      try{
         // Instantiate the message writer
         pb = new NetMessageWriter();

         // Do not employ auto-commit ... I
         // will perform explict commits after having
         // written a number of messages.
         pb.setAutoCommit(false);

         // Notify me if there is an exception in the
         // network processing thread
         pb.setExceptionListener(this);

         // Identify the remote service
         pb.setRemoteServiceName("SAFESTORE");

         // Connect to the remote service
         pb.connect();

         System.out.println("Beginning message writes");

         // write ten million messages as strings to the service
         // (can also write byte arrays)
         int i;
         int j = 0;
         for(i = 0; i < TEN_MILLION; ++i)
            {
            pb.write(message);
            j++;
            if(j == 20)
               {
               pb.commit();
               j = 0;
               }
            }

         // close the connection to the remote service
         pb.disconnect();

         }
      catch(Exception e){}
      }

    public static void main(String[] args)
          {
          TestMWriter ts = new TestMWriter();
          ts.run();
          try{System.in.read(new byte[10]);}
          catch(Exception ignore){}
          }


}






