/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.sample;

import java.util.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.IOException;
import java.text.DecimalFormat;

import osmq.mcns.*;
import osmq.util.*;
import osmq.records.DataRecord;
import osmq.mque.*;

import osmq.agents.NetReader;

   public class TestRReader implements ExceptionListener
   {
   public TestRReader(){}
   public void onException(Exception e)
      {
      System.out.println("Exception in TestSub " + e.getMessage());
      }

   public void run()
    {
    String withCommas = "###,###,###,###,###";
    DecimalFormat myformat = new DecimalFormat(withCommas);
    //String s;
    NetReader _netreader;
    int SHOW_COUNT = 100000;
    long totalLength = 0;
    EventRateMgr _erm = new EventRateMgr();
    DataRecord _datarecord;
    
    try{
       // Instantiate the network message reader
       _netreader = new NetReader();

       // Notify my onException(Exception) method if there is
       // a network error
       _netreader.setExceptionListener(this);


       // Identify my unique network service name for writers to dynamically locate
       _netreader.setServiceName("SAFESTORE");

       // Define my listening socket port
       _netreader.setPort(3843);

       // If the queue reaches 1000 records, overflow remainder to disk
       _netreader.setOverflowSize(1000);

       // Open the connection for message reception and begin receiving messages
       // in the queue
       _netreader.open();

        System.out.println("Message reader opened");

       // Get the first message as a DataRecord
       _datarecord =  _netreader.getNextDataRecord();

       // Initialize the event rate object
       _erm.reset();
       
       // process inbound messages
       for(;;)
          {
          // Capture statistic for later display
          totalLength += _datarecord.getText().length();
          _erm.incrementCount();

          // after every 10,000 records, display the current record,
          // the average rate and the total processed
          if((_erm.getCount() % SHOW_COUNT) == 0)
             {
             // Determine the number of fields in the record
             int ct = _datarecord.getElementCount();
             System.out.println("Message = " +  _datarecord.getText());
             
             // display the value of each record field as a string
             for(int i = 1; i <= ct; ++i)
                 {
                 System.out.println("Field " + i + " = " + _datarecord.getString(i));
                 }

             System.out.println("avg rate = " + _erm.getRatePerSec() + "/sec");
             System.out.println("Total count = " + myformat.format(_erm.getCount()));
             System.out.println("Bytes read = " + myformat.format(totalLength));
             System.out.println("Total pub connections = " + _netreader.getConnectionCount());
             }

          // fetch the next message as a record
          _datarecord =  _netreader.getNextDataRecord();
          }
       }

    catch(Exception e)
       {
       System.out.println("Peer Subscriber Exception: " +
                          e.getMessage());
       try{System.in.read(new byte[10]);}
       catch(Exception ignore){}
       }
    }

    public static void main(String[] args)
          {
          TestRReader ts = new TestRReader();
          ts.run();
          try{System.in.read(new byte[10]);}
          catch(Exception ignore){}
          }
    }






