/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.util;

import java.util.*;
import java.io.*;
import java.lang.System;

/**
  * Takes a delimited string and returns the elements as an
  * enumeration ( nextElement() and hasMoreElements() )
  */
final public class DelimitedStringParser implements Enumeration, Cloneable, Serializable
  {
  private char   _delimiter = '|';
  private int    _maxposition = 0;
  private int    _currentposition = 0;
  private String _str = null;

  public DelimitedStringParser(){}

  public DelimitedStringParser(String str)
     {setString(str);}

  public void setString(String str)
     {
     _currentposition = 0;
     _str = str;
     if(_str != null)
        {
        _maxposition = _str.length();
        if(_maxposition > 0)
           _delimiter = _str.charAt(0);
        }
     }

  public int getCount()
     {
     int count = 0, pos = 0;

     while(pos < _maxposition)
        {
        if(_str.charAt(pos++) == _delimiter)
           ++count;
        }
     return count;
     }

  private int getIndexOf(int item)
     {
     int count = 0, pos = 0;

     for(pos=1; pos < _maxposition && count < item; ++pos)
        {
        if(_str.charAt(pos) == _delimiter)
           ++count;
        }
     return(pos < _maxposition) ? pos : -1;
     }

  private void skipdelimiter()
     {
     if(_str.charAt(_currentposition) == _delimiter)
        _currentposition++;
     }

  public boolean hasMoreTokens()
     {return _currentposition < _maxposition;}

  public Object firstToken()
     {
     _currentposition = 0;
     return nextToken();
     }

  public Object nextToken()
     {
     if(_currentposition >= _maxposition)
       throw new NoSuchElementException();
     skipdelimiter();
     int startposition = _currentposition;
     while((_currentposition < _maxposition) &&
           (_str.charAt(_currentposition) != _delimiter))
              _currentposition++;

     return _str.substring(startposition, _currentposition);
     }

  /** for implementing Enumeration */
  public Object nextElement()
     {return nextToken();}

  public boolean hasMoreElements()
     {return hasMoreTokens();}

  public Object getToken(int posn)
       {
       int i = getIndexOf(posn);
       if(i > -1)
           {
           _currentposition = i;
           return nextToken();
           }
       else
          return null;
       }

   public Object clone()
	    {
	    try{
         DelimitedStringParser p = (DelimitedStringParser) super.clone();
         p._delimiter = _delimiter;
         return p;
         }
	    catch(CloneNotSupportedException e){return this;}
	    }
   }
