/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.util;

import java.net.*;
import java.io.*;
import java.util.*;
import java.math.*;

public class DynByteArray
   {
   final private static int SIZE_INCREMENT = 16;
   private int    _buffersz = 0;
   private byte[] _readbuffer;
   private int    _contentsize = 0;

   public DynByteArray()
      {
      this(SIZE_INCREMENT);
      }

   public DynByteArray(int sz)
      {
      _buffersz = sz;
      _readbuffer = new byte[_buffersz];
      }

   public byte[] getBuffer()
      {return _readbuffer;}

   public int getBufferSize(){return _buffersz;}

   public void clearContentSize(){setContentSize(0);}
   public int getContentSize(){return _contentsize;}
   public void setContentSize(int sz){_contentsize = sz;}
   
   public void appendByte(byte value)
      {
      ensureBufferSize(_contentsize + 1);
      _readbuffer[_contentsize++] = value;
      }

   public byte[] getDuplicate()
      {
      byte[] newbuffer = new byte[_contentsize];
         // copy old to new
      System.arraycopy(_readbuffer, 0, newbuffer, 0, _contentsize);
      return newbuffer;
      }

   public String toString()
      {
      return new String(getBuffer(), 0, _contentsize);
      }
      
   private byte[] ensureBufferSize(int newsz)
      {
      if(_buffersz < newsz)
         {
         // allocate a new buffer
         byte[] newbuffer = new byte[newsz + SIZE_INCREMENT];
         // copy old to new
         System.arraycopy(_readbuffer, 0, newbuffer, 0, _buffersz);
         // new size variable value
         _buffersz = newsz + SIZE_INCREMENT;
         // buffer refers to new
         _readbuffer = newbuffer;
         }
      return _readbuffer;
      }

   public void initializeBuffer(int newsz)
      {
      if(_buffersz < newsz)
         {
         _buffersz = newsz + SIZE_INCREMENT;
         _readbuffer = new byte[_buffersz];
         }
      _contentsize = newsz;
      }

   }
