/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.util;

/**
  * A class that tracks time duration and number of events.
  * It is used to calculate the rate of activity of the events
  * that it tracks.
  */
public class EventRateMgr
  {
  private StopWatch _watch = new StopWatch();
  private long      _events = 0;

  public EventRateMgr()
     {}

  public void incrementCount(){_events++;}

  public void incrementCountBy(int value)
     {_events += value;}

  /**
    * Resets the time duration to zero and the number of
    * events to zero.
    */
  public void reset()
     {
     _events = 0;
     _watch.reset();
     }
  /**
    * Returns the number of events that have occured since
    * the manager was instantiated or the manager was reset.
    */
  public long getCount(){return _events;}

  public int getRatePerSec()
       {
       int secs = _watch.getDurationSeconds();
       return(secs > 0 && _events > 0) ? (int)(_events /secs) : 0;
       }

  }
