/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 

package osmq.util;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Date;

public class Formatter
  {
  private static GregorianCalendar calendar;


  /**
    * Removes leading zeros and trims spaces from an interger string.
    */
  public static String formatInteger(String value)
     {
     if(value == null || value.length() < 2)
         return value;
     value = value.trim();
     int posn = 0;
     int lgh = value.length();
     while(posn < lgh && value.charAt(posn) == '0')
        ++posn;
     return value.substring(posn, lgh);
     }

  private static boolean isNumber(char v)
     {
     return v >= '0' && v <= '9';
     }

  public static String trimDate2(String source)
     {
     if(source == null || source.length() == 0)
        return source;
     int ctr  = 0;
     int posn = 0;
     int lgh  = source.length();
     StringBuffer result = new StringBuffer(lgh);
     result.setLength(lgh);
     while(posn < lgh)
        {
        char c = source.charAt(posn);
        if(isNumber(c))
           result.setCharAt(ctr++, c);
        ++posn;
        }
     return result.toString();
     }

   /**
     * Returns a date string in CCYYMMDD format
     */
  public static synchronized String javaDateToString(Date value)
      {
      GregorianCalendar calendar = getCalendar();
      calendar.setTime(value);
      return new String ("" +
                       calendar.get(Calendar.YEAR) +
                       calendar.get(Calendar.MONTH) +
                       calendar.get(Calendar.DAY_OF_MONTH)
                       );
      }

   /**
     * Converts a date string in CCYYMMDD format to a Date object
     */
    public static java.util.Date stringToDate(String ccyymmdd)
       {
  	   int year;
	     int month;
	     int day;

       if (ccyymmdd == null || ccyymmdd.length() != 8)
          throw new java.lang.IllegalArgumentException("Illegal date string");

       year = Integer.parseInt(ccyymmdd.substring(0, 4));
       month = Integer.parseInt(ccyymmdd.substring(4, 6)) - 1;
       day = Integer.parseInt(ccyymmdd.substring(6));
       return new GregorianCalendar(year, month, day).getTime();
       }

  public static String unformatDate(String rawdate)
     {
     if(rawdate == null || rawdate.length() == 0)
        return rawdate;
        
     char[] buffer = new char[rawdate.length()];
     int ctr  = 0;
     int posn = 0;
     int lgh  = rawdate.length();
     while(posn < lgh)
        {
        char c = rawdate.charAt(posn);
        if(isNumber(c))
           buffer[ctr++] = c;
        ++posn;
        }
     return new String(buffer, 0, ctr);
     }


    /**
      *  Converts an <i>unformatted</i> date in "ccyymmdd" format
      *  to a formatted date in "ccyy-mm-dd" format (adding hyphens).
      *  This results in a date with the standard ISO SQL date string format.
      */
    public static String formatDateString(String rawdate)
       {
       if(rawdate != null && rawdate.length() == 8)
          {
          return rawdate.substring(0,4) + '-' +
          rawdate.substring(4,6) + '-' +
          rawdate.substring(6,8);
          }
       else
          return rawdate;
       }

  /**
    *  Converts a <i>formatted</i> date in "ccyy-mm-dd" format
    *  to "ccyymmdd" format (removing the two hyphens).
    */
  public static String unformatDateString(String rawdate)
     {
     if(rawdate != null && rawdate.length() == 10)
        {
        String ccyy = rawdate.substring(0,4);
        String mm = rawdate.substring(5,7);
        String dd = rawdate.substring(8,10);
        return ccyy+mm+dd;
        }
     else
        return "";
     }

  public static String unformatDecimalString(String value)
     {
     int lgh  = value.length();
     StringBuffer result = new StringBuffer(lgh);
     result.setLength(lgh);
     int i = 0;
     int ctr = 0;
     while(i < lgh)
        {
        char c = value.charAt(i);
        if(isNumber(c) || c == '.')
           result.setCharAt(ctr++, c);
        ++i;
        }
     return result.toString();
     }

  public static java.sql.Date formatSQLDateString(String rawdate)
     {
     return java.sql.Date.valueOf(formatDateString(rawdate));
     }

  private static GregorianCalendar getCalendar()
     {
     if (calendar == null)
        calendar = new GregorianCalendar();
     return calendar;
     }


  public static String formatDecimals(String value)
      {
      if(value == null || value.length() == 0)
         return value;
      value = value.trim();
      int dposn = value.indexOf('.', 0);

      // test if a decimal point was found
      if(dposn > -1)
         {
         // number of chars to the right of the decimal point
         int dcount = value.length() - dposn - 1;

         // trim non-significant zeros
         while(value.charAt(dposn + dcount ) == '0' && dcount > 1)
            {
            --dcount;
            }
             
         return value.substring(0, dposn + dcount + 1);
         }
         
      // no decimal point was found, so add one
      return new String(value + ".0");
      }
      
  public static void main(String[] vals)
     {
     System.out.println(formatDecimals("1.304"));
     System.out.println(formatDecimals("0.000"));
     System.out.println(formatInteger("0345"));
     System.out.println(formatDecimals("1567823.000000"));
     System.out.println(formatInteger("1"));
     System.out.println(formatInteger("0"));

     System.out.println(unformatDate("2000-05-07"));
     System.out.println(formatDateString("20000507"));
     System.out.println(trimDate2("2000-05-07"));
     System.out.println(formatSQLDateString("20000507"));
     }
  } 