/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.util;

import osmq.logs.Logs;

public class StringParser
    {
    private String _string;
    private int    _position = 0;
    private int    _occurance = -1;

    private int    _foundcount = 0;
    private int    _lgh;

    public StringParser(String string)
       {
       _string = string;
       _lgh = (string != null) ? string.length() : 0;
       }

    public final int getPosition()
       {
       return _position;
       }

    public boolean validString()
       {
       return _lgh != 0;
       }
       
    public int getOccurance()
       {
       return _occurance;
       }

    private static boolean isWhiteSpace(char value)
       {
       switch(value)
          {
          case ' ':
          case '\t':
             return true;
          default:
             return false;
          }
       }

   final public boolean findSubstring(String sub)
      {
      int v = _string.indexOf(sub);
      if(v > -1)
         {
         _position = v;
         return true;
         }
      return false;
      }
   final public boolean ignoreSubstring(String sub)
      {
      if(findSubstring(sub))
         {
         _position += sub.length();
         return true;
         }
      return false;
      }
      
    /**
      * Locates an occurance of a character within a string that matches
      * a character in another string, and returns its offset.
      * The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
 
    public final boolean findTokenOccurance(String token, int occurance)
     {
     int oldpos = _position;
     _occurance  = occurance;
     _foundcount = 0;

     while(_position < _lgh)
         {
         if(hasChar(token, _string.charAt(_position)))
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               {
               return true;           // return its zero-relative offset postion
               }
            }
         _position++;
         }
     _position = oldpos;
     return false;
     }

    public void resetPosition()
       {
       _position = 0;
       }

    private static boolean hasChar(String s, char c)
       {
       for(int i = 0; i < s.length(); ++i)
          {
          if(s.charAt(i) == c)
             return true;
          }
       return false;
       }
       
    /**
      * Locates an occurance of a character within a string and
      * returns its offset.  The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
    public final boolean findTokenOccurance(char token, int occurance)
     {
     int oldpos = _position;
     
     _occurance  = occurance;
     _foundcount = 0;

     while(_position < _lgh)
         {
         if(_string.charAt(_position) == token)
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               return true; // return its zero-relative offset postion
            }
         _position++;
         }
     _position = oldpos;
     return false;
     }
   } 