/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.util;


import java.util.*;

public class TickEventBean  implements Runnable
  {
  private Thread t;
  private boolean _canContinue = true;
  private boolean _demonpriority = false;
  private int     _secs = 5;
  private int     _priority = Thread.NORM_PRIORITY;

  private Vector listenerList = new Vector();
  public TickEventBean() {}

  public void setDemonPriority(boolean b)
     {_demonpriority = true;}

  public boolean isDemonPriority()
     {return _demonpriority;}

  public void setPriority(int priority)
     {
     _priority = priority;
     }

  public int getPriority()
     {
     return _priority;
     }

  public synchronized void addTickEventListener(TickEventListener l)
    {listenerList.addElement(l);}

  public synchronized void removeTickEventListener(TickEventListener l)
    {listenerList.removeElement(l);}

  public void begin()
     {
     _canContinue = true;
     t = new Thread(this);
     if(isDemonPriority())
        t.setDaemon(true);
     else
        t.setPriority(getPriority());
     t.start();
     }

  public boolean isRunning()
     {return t != null;}

  public void stop()
     {
     _canContinue = false;
     t = null;
     }

  final public void run()
     {
     while(_canContinue)
        {
        // go to sleep for number of seconds
        try {Thread.sleep(_secs * 1000);}
        catch(InterruptedException e){}

        if(_canContinue)
           fireTickEvent();
        }
     }

  public void setAlertInterval(int secs)
    {
    if(secs > 0)
       _secs = secs;
    }

  public int getAlertInterval()
    {return _secs;}

  protected void fireTickEvent()
    {processTickEvent (new TickEvent (this, TickEvent.TICK));}

  protected void processTickEvent(TickEvent e)
    {
    switch (e.getID())
      {
      case TickEvent.TICK:
        for (int i=0; i < listenerList.size(); i++)
          ((TickEventListener)listenerList.elementAt(i)).onTickEvent(e);
        break;
       }
    }
  }

