/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.util;

import osmq.logs.Logs;

public class TokenFinder
    {
    int _lastfound = -1;
    int _occurance = -1;
    int _offset = 0;
    int _foundcount = 0;
    int _lgh = 0;

    public TokenFinder()
       {
       }

    public final int getLastFound()
       {
       return _lastfound;
       }

    public int getOccurance()
       {
       return _occurance;
       }

    final public boolean wasFound()
      {
      return _lastfound > -1;
      }


      /**
      * Locates an occurance of a byte within a byte array and
      * returns its offset.  The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
    public final int findTokenOccurance(byte[] s, byte token, int occurance)
     {
     _occurance  = occurance;
     _lastfound  = -1;
     _foundcount = 0;
     _offset     = 0;
     _lgh = s.length;

     while(_offset < _lgh)
         {
         if(s[_offset] == token)
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               {
               _lastfound = _offset;
               return _offset;           // return its zero-relative offset postion
               }
            }
         _offset++;
         }
     return -1;
     }


    /**
      * Locates an occurance of a character within a string that matches
      * a character in another string, and returns its offset.
      * The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
 
    public final int findTokenOccurance(String s, String token, int occurance)
     {
     _occurance  = occurance;
     _lastfound  = -1;
     _foundcount = 0;
     _offset     = 0;
     _lgh = s.length();

     while(_offset < _lgh)
         {
         if(hasChar(token, s.charAt(_offset)))
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               {
               _lastfound = _offset;
               return _offset;           // return its zero-relative offset postion
               }
            }
         _offset++;
         }
     return -1;
     }

    //public int findSubstring(String 

    private boolean hasChar(String s, char c)
       {
       for(int i = 0; i < s.length(); ++i)
          {
          if(s.charAt(i) == c)
             return true;
          }
       return false;
       }
    /**
      * Locates an occurance of a character within a string and
      * returns its offset.  The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
    public final int findTokenOccurance(String s, char token, int occurance)
     {
     _occurance  = occurance;
     _lastfound  = -1;
     _foundcount = 0;
     _offset     = 0;
     _lgh = s.length();

     while(_offset < _lgh)
         {
         if(s.charAt(_offset) == token)
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               {
               _lastfound = _offset;
               return _offset;           // return its zero-relative offset postion
               }
            }
         _offset++;
         }
     return -1;
     }
   } 